@@ -235,7 +235,7 @@ const struct ipset_envopts ipset_envopts[] = {
 	{ .name = { "-o", "-output" },
 	  .has_arg = IPSET_MANDATORY_ARG,	.flag = IPSET_OPT_MAX,
 	  .parse = ipset_parse_output,
-	  .help = "plain|save|xml\n"
+	  .help = "plain|save|xml|json\n"
 		  "       Specify output mode for listing sets.\n"
 		  "       Default value for \"list\" command is mode \"plain\"\n"
 		  "       and for \"save\" command is mode \"save\".",
@@ -429,6 +429,8 @@ ipset_parse_output(struct ipset *ipset,
 		return ipset_session_output(session, IPSET_LIST_PLAIN);
 	else if (STREQ(str, "xml"))
 		return ipset_session_output(session, IPSET_LIST_XML);
+	else if (STREQ(str, "json"))
+		return ipset_session_output(session, IPSET_LIST_JSON);
 	else if (STREQ(str, "save"))
 		return ipset_session_output(session, IPSET_LIST_SAVE);
 
@@ -1876,9 +1878,6 @@ static int ipset_xlate(struct ipset *ipset, enum ipset_cmd cmd,
 				cmd == IPSET_CMD_DEL ? "delete" : "get",
 		       ipset_xlate_family(family), table, set);
 
-		typename = ipset_data_get(data, IPSET_OPT_TYPENAME);
-		type = ipset_xlate_set_type(typename);
-
 		xlate_set = (struct ipset_xlate_set *)
 				ipset_xlate_set_get(ipset, set);
 		if (xlate_set && xlate_set->interval)
@@ -1999,7 +1998,7 @@ static int ipset_xlate_restore(struct ipset *ipset)
 
 		ret = build_argv(ipset, c);
 		if (ret < 0)
-			return ret;
+			break;
 
 		cmd = ipset_parser(ipset, ipset->newargc, ipset->newargv);
 		if (cmd < 0)
